@@ -30,6 +30,9 @@
 from __future__ import division
 import math
 
+import logging
+logger = logging.getLogger('root')
+
 
 class FrameList(object):
     """
@@ -163,6 +166,10 @@ class BoardUnitList(object):
         return len(self._list)
 
 
+def normalizeValueTable(table):
+    return {int(k): v for k, v in table.items()}
+
+
 class Signal(object):
     """
     contains on Signal of canmatrix-object
@@ -178,89 +185,61 @@ class Signal(object):
 
     def __init__(self, name, **kwargs):
 
-        if 'startBit' in kwargs:
-            self._startbit = int(kwargs["startBit"])
-        else:
-            self._startbit = 0
-
-        if 'signalSize' in kwargs:
-            self._signalsize = int(kwargs["signalSize"])
-        else:
-            self._signalsize = 0
-
-        if 'is_little_endian' in kwargs:
-            self._is_little_endian = kwargs["is_little_endian"]
-        else:
-            self._is_little_endian = True
-
-        if 'is_signed' in kwargs:
-            self._is_signed = kwargs["is_signed"]
-        else:
-            self._is_signed = True
-
-        if 'is_float' in kwargs:
-            self._is_float = kwargs["is_float"]
-        else:
-            self._is_float = False
-
-        if 'factor' in kwargs:
-            self._factor = float(kwargs["factor"])
-        else:
-            self._factor = float(1)
-
-        if 'offset' in kwargs:
-            self._offset = float(kwargs["offset"])
-        else:
-            self._offset = float(0)
-
-        if 'unit' in kwargs:
-            self._unit = kwargs["unit"]
-        else:
-            self._unit = ""
-
-        if 'receiver' in kwargs:
-            self._receiver = kwargs["receiver"]
-        else:
-            self._receiver = []
-
-        if 'comment' in kwargs:
-            self._comment = kwargs["comment"]
-        else:
-            self._comment = None
-
-        if 'multiplex' in kwargs:
-            if kwargs["multiplex"] is not None and kwargs[
-                    "multiplex"] != 'Multiplexor':
-                multiplex = int(kwargs["multiplex"])
+        def multiplex(value):
+            if value is not None and value != 'Multiplexor':
+                multiplex = int(value)
             else:
-                multiplex = kwargs["multiplex"]
-            self._multiplex = multiplex
-        else:
-            self._multiplex = None
+                multiplex = value
+            return multiplex
+                        
 
-        if 'min' in kwargs:
-            min = kwargs["min"]
-        else:
-            min = None
+        args = [
+            ('startBit', '_startbit', int, 0),
+            ('signalSize', '_signalsize', int, 0),
+            ('is_little_endian', '_is_little_endian', bool, True),
+            ('is_signed', '_is_signed', bool, True),
+            ('factor', '_factor', float, 1),
+            ('offset', '_offset', float, 0),
+            ('min', '_min', float, None),
+            ('max', '_max', float, None),
+            ('unit', '_unit', None, ""),
+            ('receiver', '_receiver', None, []),
+            ('comment', '_comment', None, None),
+            ('multiplex', '_multiplex', multiplex, None),
+            ('is_float', '_is_float', bool, False),
+            ('enumeration', 'enumeration', str, None),
+            ('comments', 'comments', None, {}),
+            ('attributes', '_attributes', None, {}),
+            ('values', '_values', None, {}),
+        ]
 
-        if min is None:
+        for arg_name, destination, function, default in args:
+            try:
+                value = kwargs[arg_name]
+            except KeyError:
+                value = default
+            else:
+                kwargs.pop(arg_name)
+            if function is not None and value is not None:
+                value = function(value)
+            setattr(self, destination, value)
+            
+        if len(kwargs) > 0:
+            raise TypeError('{}() got unexpected argument{} {}'.format(
+                self.__class__.__name__,
+                's' if len(kwargs) > 1 else '',
+                ', '.join(kwargs.keys())
+            ))
+
+
+        # be shure to calc min/max after parsing all arguments 
+        if self._min is None:
             self.setMin()
-        else:
-            self._min = float(min)
 
-        if 'max' in kwargs:
-            max = kwargs["max"]
-        else:
-            max = None
-
-        if max is None:
+        if self._max is None:
             self.setMax()
-        else:
-            self._max = float(max)
 
         self._name = name
-        self._attributes = {}
-        self._values = {}
 
     @property
     def name(self):
@@ -288,7 +267,7 @@ class Signal(object):
 
     @values.setter
     def values(self, valueTable):
-        self._values = valueTable
+        self._values = normalizeValueTable(valueTable)
 
     @property
     def comment(self):
@@ -325,6 +304,10 @@ class Signal(object):
     @property
     def is_float(self):
         return self._is_float
+    
+    @is_float.setter
+    def is_float(self, value):
+        self._is_float = value
 
     @property
     def is_signed(self):
@@ -372,7 +355,7 @@ class Signal(object):
         Add Attribute to Signal
         """
         if attribute not in self._attributes:
-            self._attributes[attribute] = value.replace('"', '')
+            self._attributes[attribute] = value
 
     def delAttribute(self, attribute):
         """
@@ -434,15 +417,27 @@ class Signal(object):
     def setMin(self, min=None):
         self._min = min
         if self._min is None:
-            rawMin = self.calculateRawRange()[0]
-            self._min = self._offset + (rawMin * self._factor)
+            self._min = self.calcMin()
+
+        return self._min
+
+    def calcMin(self):
+        rawMin = self.calculateRawRange()[0]
+
+        return self._offset + (rawMin * self._factor)
 
     def setMax(self, max=None):
         self._max = max
 
         if self._max is None:
-            rawMax = self.calculateRawRange()[1]
-            self._max = self._offset + (rawMax * self._factor)
+            self._max = self.calcMax()
+
+        return self._max
+            
+    def calcMax(self):
+        rawMax = self.calculateRawRange()[1]
+
+        return self._offset + (rawMax * self._factor)
 
     def __str__(self):
         return self._name
@@ -514,39 +509,37 @@ class Frame(object):
 
     def __init__(self, name, **kwargs):
         self._name = name
-        if 'Id' in kwargs:
-            self._Id = int(kwargs["Id"])
-        else:
-            self._Id = 0
+            
+        args = [
+            ('Id', '_Id', int, 0),
+            ('dlc', '_Size', int, 0),
+            ('transmitter', '_Transmitter', None, []),
+            ('extended', '_extended', bool, False),
+            ('comment', '_comment', str, None),
+            ('signals', '_signals', None, []),
+            ('mux_names', '_mux_names', None, {}),
+            ('attributes', '_attributes', None, {}),
+            ('receiver', '_receiver', None, []),
+            ('SignalGroups', '_SignalGroups', None, []),
+        ]
 
-        if 'dlc' in kwargs:
-            self._Size = int(kwargs["dlc"])
-        else:
-            self._Size = 0
-
-        if 'transmitter' in kwargs:
-            self._Transmitter = [kwargs["transmitter"]]
-        else:
-            self._Transmitter = []
-
-        if 'extended' in kwargs:
-            self._extended = kwargs["extended"]
-        else:
-            self._extended = 0
-
-        if 'comment' in kwargs:
-            self._comment = kwargs["comment"]
-        else:
-            self._comment = None
-
-        if 'signals' in kwargs:
-            self._signals = kwargs["signals"]
-        else:
-            self._signals = []
-
-        self._attributes = {}
-        self._receiver = []
-        self._SignalGroups = []
+        for arg_name, destination, function, default in args:
+            try:
+                value = kwargs[arg_name]
+            except KeyError:
+                value = default
+            else:
+                kwargs.pop(arg_name)
+            if function is not None and value is not None:
+                value = function(value)
+            setattr(self, destination, value)
+            
+        if len(kwargs) > 0:
+            raise TypeError('{}() got unexpected argument{} {}'.format(
+                self.__class__.__name__,
+                's' if len(kwargs) > 1 else '',
+                ', '.join(kwargs.keys())
+            ))
 
     @property
     def attributes(self):
@@ -568,6 +561,10 @@ class Frame(object):
     def transmitter(self):
         return self._Transmitter
 
+    @transmitter.setter
+    def transmitter(self, value):
+        self._Transmitter = value
+
     @property
     def size(self):
         return self._Size
@@ -596,6 +593,14 @@ class Frame(object):
     def extended(self, value):
         self._extended = value
 
+    @property
+    def mux_names(self):
+        return self._mux_names
+
+    @mux_names.setter
+    def mux_names(self, value):
+        self._mux_names = value
+
     @property
     def name(self):
         return self._name
@@ -686,6 +691,64 @@ class Frame(object):
             if sig.getStartbit() + int(sig._signalsize) > maxBit:
                 maxBit = sig.getStartbit() + int(sig._signalsize)
         self._Size = max(self._Size, int(math.ceil(maxBit / 8)))
+        
+    def findNotUsedBits(self):
+        """
+        find unused bits in frame
+        return dict with position and length-tuples
+        """
+        bitfield = []
+        bitfieldLe = []
+        bitfieldBe = []
+        
+        for i in range(0,64):
+            bitfieldBe.append(0)
+            bitfieldLe.append(0)
+            bitfield.append(0)
+        i = 0
+
+        for sig in self._signals:
+            i += 1
+            for bit in range(sig.getStartbit(),  sig.getStartbit() + int(sig._signalsize)):
+                if sig._is_little_endian:
+                    bitfieldLe[bit] = i
+                else:
+                    bitfieldBe[bit] = i
+
+        for i in range(0,8):
+            for j in range(0,8):
+                bitfield[i*8+j] = bitfieldLe[i*8+(7-j)]
+
+        for i in range(0,8):
+            for j in range(0,8):
+                if bitfield[i*8+j] == 0:
+                    bitfield[i*8+j] = bitfieldBe[i*8+j]
+        
+
+        return bitfield
+    
+    def createDummySignals(self):
+        bitfield = self.findNotUsedBits()
+#        for i in range(0,8):
+#            print (bitfield[(i)*8:(i+1)*8])
+        startBit = -1
+        sigCount = 0
+        for i in range(0,64):
+            if bitfield[i] == 0 and startBit == -1:
+                startBit = i
+            if (i == 63 or bitfield[i] != 0) and startBit != -1:
+                if i == 63:
+                    i = 64
+                self.addSignal(Signal("_Dummy_%s_%d" % (self.name,sigCount),signalSize=i-startBit, startBit=startBit, is_little_endian = False))
+                startBit = -1
+                sigCount +=1
+                
+#        bitfield = self.findNotUsedBits()
+#        for i in range(0,8):
+#            print (bitfield[(i)*8:(i+1)*8])
+                
+            
+                
 
     def updateReceiver(self):
         """
@@ -806,7 +869,7 @@ class CanMatrix(object):
         return iter(self._fl)
 
     def addValueTable(self, name, valueTable):
-        self._valueTables[name] = valueTable
+        self._valueTables[name] = normalizeValueTable(valueTable)
 
     def addAttribute(self, attribute, value):
         """
@@ -1001,50 +1064,45 @@ class CanMatrix(object):
 #
 #
 #
-def putSignalValueInFrame(startbit, len, format, value, frame):
+def computeSignalValueInFrame(startbit, ln, fmt, value):
     """
-    puts a signal-value to the right position in a frame
+    compute the signal value in the frame
     """
+    import pprint
 
-    if format == 1:  # Intel
-        lastbit = startbit + len
-        firstbyte = math.floor(startbit / 8) - 1
-        lastbyte = math.floor((lastbit - 1) / 8)
-        # im lastbyte mit dem msb anfangen
-        # im firstbyte mit dem lsb aufhoeren
-        for i in range(lastbyte, firstbyte, -1):
-            if lastbit % 8 != 0:
-                nbits = lastbit % 8
-            else:
-                nbits = min(len, 8)
-            nbits = min(len, nbits)
+    frame = 0
+    if fmt == 1:  # Intel
+    # using "sawtooth bit counting policy" here
+        pos = ((7 - (startbit % 8)) + 8*(int(startbit/8)))
+        while ln > 0:
+            # How many bits can we stuff in current byte?
+            #  (Should be 8 for anything but the first loop)
+            availbitsInByte = 1 + (pos % 8)
+            # extract relevant bits from value
+            valueInByte = value & ((1<<availbitsInByte)-1)
+            # stuff relevant bits into frame at the "corresponding inverted bit"
+            posInFrame = ((7 - (pos % 8)) + 8*(int(pos/8)))
+            frame |= valueInByte << posInFrame
+            # move to the next byte
+            pos += 0xf
+            # discard used bytes
+            value = value >> availbitsInByte
+            # reduce length by how many bits we consumed
+            ln -= availbitsInByte
 
-            start = lastbit - 1 - int(math.floor((lastbit - 1) / 8)) * 8
-            end = lastbit - nbits - int(math.floor((lastbit - nbits) / 8)) * 8
-
-            len -= nbits
-            mask = (0xff >> 7 - start) << end
-            mask &= 0xff
-            frame[i] |= (((value >> len) << end) & mask)
-            lastbit = startbit + len
     else:  # Motorola
-        # TODO needs review, is probably wrong till we use LSB for startbit
-        firstbyte = math.floor(startbit / 8)
-        bitsInfirstByte = startbit % 8 + 1
-        restnBits = len - bitsInfirstByte
-        lastbyte = firstbyte + math.floor(restnBits / 8)
-        if restnBits % 8 > 0:
-            lastbyte += 1
-        restLen = len
-        nbits = bitsInfirstByte
-        for i in range(firstbyte, lastbyte + 1):
-            end = 0
-            if restLen < 8:
-                end = 8 - restLen
-            mask = (0xff >> (8 - nbits)) << end
-            restLen -= nbits
-            frame[i] |= ((value >> restLen) << end) & mask
-            nbits = min(restLen, 8)
+        # Work this out in "sequential bit counting policy"
+        # Compute the LSB position in "sequential"
+        lsbpos = ((7 - (startbit % 8)) + 8*(int(startbit/8)))
+        # deduce the MSB position
+        msbpos = 1 + lsbpos - ln
+        # "reverse" the value
+        cvalue = int(format(value, 'b')[::-1],2)
+        # shift the value to the proper position in the frame
+        frame = cvalue << msbpos
+
+    # Return frame, to be accumulated by caller
+    return frame
 
 
 class CanId(object):
