@@ -26,6 +26,7 @@ from __future__ import division
 from __future__ import print_function
 from __future__ import absolute_import
 
+import collections
 import logging
 logger = logging.getLogger('root')
 
@@ -49,6 +50,18 @@ def normalizeName(name, whitespaceReplacement):
     return name
 
 
+def format_float(f):
+    s = str(f).upper()
+    if s.endswith('.0'):
+        s = s[:-2]
+
+    if 'E' in s:
+        s = s.split('E')
+        s = '%sE%s%s' % (s[0], s[1][0], s[1][1:].rjust(3, '0'))
+
+    return s.upper()
+
+
 def dump(db, f, **options):
     if 'dbcExportEncoding' in options:
         dbcExportEncoding = options["dbcExportEncoding"]
@@ -65,6 +78,10 @@ def dump(db, f, **options):
             print("Warning: Settings may result in whitespace in DBC variable names.  This is not supported by the DBC format.")
     else:
         whitespaceReplacement = '_'
+    if 'writeValTable' in options:
+        writeValTable = options["writeValTable"]
+    else:
+        writeValTable = True
 
     f.write("VERSION \"created by canmatrix\"\n\n".encode(dbcExportEncoding))
     f.write("\n".encode(dbcExportEncoding))
@@ -79,27 +96,51 @@ def dump(db, f, **options):
         f.write((bu.name + " ").encode(dbcExportEncoding))
     f.write("\n\n".encode(dbcExportEncoding))
 
-    # ValueTables
-    for table in db.valueTables:
-        f.write(("VAL_TABLE_ " + table).encode(dbcExportEncoding))
-        for row in db.valueTables[table]:
-            f.write(
-                (' ' +
-                 row +
-                 ' "' +
-                 db.valueTables[table][row] +
-                 '"').encode(dbcExportEncoding))
-        f.write(";\n".encode(dbcExportEncoding))
-    f.write("\n".encode(dbcExportEncoding))
+    if writeValTable:
+        # ValueTables
+        for table in db.valueTables:
+            f.write(("VAL_TABLE_ " + table).encode(dbcExportEncoding))
+            for row in db.valueTables[table]:
+                f.write(
+                    (' ' +
+                     str(row) +
+                     ' "' +
+                     db.valueTables[table][row] +
+                     '"').encode(dbcExportEncoding))
+            f.write(";\n".encode(dbcExportEncoding))
+        f.write("\n".encode(dbcExportEncoding))
+
+    output_names = collections.defaultdict(dict)
+
+    for frame in db.frames:
+        normalized_names = collections.OrderedDict((
+            (s, normalizeName(s.name, whitespaceReplacement))
+            for s in frame.signals
+        ))
+
+        duplicate_signal_totals = collections.Counter(normalized_names.values())
+        duplicate_signal_counter = collections.Counter()
+
+        numbered_names = collections.OrderedDict()
+
+        for signal in frame.signals:
+            name = normalized_names[signal]
+            duplicate_signal_counter[name] += 1
+            if duplicate_signal_totals[name] > 1:
+                # TODO: pad to 01 in case of 10+ instances, for example?
+                name += str(duplicate_signal_counter[name] - 1)
+
+            output_names[frame][signal] = name
 
     # Frames
     for bo in db.frames:
+        multiplex_written = False
         if bo.transmitter.__len__() == 0:
             bo.addTransmitter("Vector__XXX")
 
         if bo.extended == 1:
             bo.id += 0x80000000
-
+        
         f.write(
             ("BO_ %d " %
              bo.id +
@@ -108,11 +149,18 @@ def dump(db, f, **options):
              bo.size +
              bo.transmitter[0] +
              "\n").encode(dbcExportEncoding))
+        duplicate_signal_totals = collections.Counter(
+            normalizeName(s.name, whitespaceReplacement) for s in bo.signals
+        )
+        duplicate_signal_counter = collections.Counter()
         for signal in bo.signals:
-            name = normalizeName(signal.name, whitespaceReplacement)
-            f.write((" SG_ " + name).encode(dbcExportEncoding))
+            if signal.multiplex == 'Multiplexor' and multiplex_written:
+                continue
+
+            f.write((" SG_ " + output_names[bo][signal]).encode(dbcExportEncoding))
             if signal.multiplex == 'Multiplexor':
                 f.write(' M '.encode(dbcExportEncoding))
+                multiplex_written = True
             elif signal.multiplex is not None:
                 f.write((" m%d " %
                          int(signal.multiplex)).encode(dbcExportEncoding))
@@ -130,14 +178,16 @@ def dump(db, f, **options):
                   signal.is_little_endian,
                   sign)).encode(dbcExportEncoding))
             f.write(
-                (" (%g,%g)" %
-                 (signal.factor, signal.offset)).encode(dbcExportEncoding))
+                (" (%s,%s)" %
+                 (format_float(signal.factor), format_float(signal.offset))).encode(dbcExportEncoding))
             f.write(
                 (" [{}|{}]".format(
-                    signal.min,
-                    signal.max)).encode(dbcExportEncoding))
+                    format_float(signal.min),
+                    format_float(signal.max))).encode(dbcExportEncoding))
             f.write(' "'.encode(dbcExportEncoding))
 
+            if signal.unit is None:
+                signal.unit = ""
             f.write(signal.unit.encode(dbcExportEncoding))
             f.write('" '.encode(dbcExportEncoding))
             if signal.receiver.__len__() == 0:
@@ -165,7 +215,7 @@ def dump(db, f, **options):
             f.write(
                 bo.comment.replace(
                     '"',
-                    '\\"').encode(dbcExportCommentEncoding))
+                    '\\"').encode(dbcExportCommentEncoding, 'ignore'))
             f.write('";\n'.encode(dbcExportEncoding))
     f.write("\n".encode(dbcExportEncoding))
 
@@ -173,17 +223,18 @@ def dump(db, f, **options):
     for bo in db.frames:
         for signal in bo.signals:
             if signal.comment is not None and signal.comment.__len__() > 0:
-                name = normalizeName(signal.name, whitespaceReplacement)
+                name = output_names[bo][signal]
                 f.write(
                     ("CM_ SG_ " +
                      "%d " %
                      bo.id +
                      name +
-                     ' "').encode(dbcExportEncoding))
+                     ' "').encode(dbcExportEncoding, 'ignore'))
                 f.write(
-                    signal.comment.replace(
-                        '"', '\\"').encode(dbcExportCommentEncoding))
-                f.write('";\n'.encode(dbcExportEncoding))
+                        signal.comment.replace(
+                            '"', '\\"').encode(dbcExportCommentEncoding, 'ignore'))
+                f.write('";\n'.encode(dbcExportEncoding, 'ignore'))
+
     f.write("\n".encode(dbcExportEncoding))
 
     # boarUnit comments
@@ -196,54 +247,54 @@ def dump(db, f, **options):
                  bu.comment.replace(
                      '"',
                      '\\"') +
-                    '";\n').encode(dbcExportCommentEncoding))
+                    '";\n').encode(dbcExportCommentEncoding,'ignore'))
     f.write("\n".encode(dbcExportEncoding))
 
     defaults = {}
-    for (type, define) in sorted(list(db.frameDefines.items())):
+    for (dataType, define) in sorted(list(db.frameDefines.items())):
         f.write(
             ('BA_DEF_ BO_ "' +
-             type +
+             dataType +
              '" ').encode(dbcExportEncoding) +
             define.definition.encode(
                 dbcExportEncoding,
                 'replace') +
             ';\n'.encode(dbcExportEncoding))
-        if type not in defaults and define.defaultValue is not None:
-            defaults[type] = define.defaultValue
-    for (type, define) in sorted(list(db.signalDefines.items())):
+        if dataType not in defaults and define.defaultValue is not None:
+            defaults[dataType] = define.defaultValue
+    for (dataType, define) in sorted(list(db.signalDefines.items())):
         f.write(
             ('BA_DEF_ SG_ "' +
-             type +
+             dataType +
              '" ').encode(dbcExportEncoding) +
             define.definition.encode(
                 dbcExportEncoding,
                 'replace') +
             ';\n'.encode(dbcExportEncoding))
-        if type not in defaults and define.defaultValue is not None:
-            defaults[type] = define.defaultValue
-    for (type, define) in sorted(list(db.buDefines.items())):
+        if dataType not in defaults and define.defaultValue is not None:
+            defaults[dataType] = define.defaultValue
+    for (dataType, define) in sorted(list(db.buDefines.items())):
         f.write(
             ('BA_DEF_ BU_ "' +
-             type +
+             dataType +
              '" ').encode(dbcExportEncoding) +
             define.definition.encode(
                 dbcExportEncoding,
                 'replace') +
             ';\n'.encode(dbcExportEncoding))
-        if type not in defaults and define.defaultValue is not None:
-            defaults[type] = define.defaultValue
-    for (type, define) in sorted(list(db.globalDefines.items())):
+        if dataType not in defaults and define.defaultValue is not None:
+            defaults[dataType] = define.defaultValue
+    for (dataType, define) in sorted(list(db.globalDefines.items())):
         f.write(
             ('BA_DEF_ "' +
-             type +
+             dataType +
              '" ').encode(dbcExportEncoding) +
             define.definition.encode(
                 dbcExportEncoding,
                 'replace') +
             ';\n'.encode(dbcExportEncoding))
-        if type not in defaults and define.defaultValue is not None:
-            defaults[type] = define.defaultValue
+        if dataType not in defaults and define.defaultValue is not None:
+            defaults[dataType] = define.defaultValue
 
     for define in sorted(defaults):
         f.write(
@@ -283,44 +334,58 @@ def dump(db, f, **options):
     f.write("\n".encode(dbcExportEncoding))
 
     # messages-attributes:
-    for bo in db.frames:
-        for attrib, val in sorted(bo.attributes.items()):
+    for frame in db.frames:
+        for attrib, val in sorted(frame.attributes.items()):
             if db.frameDefines[attrib].type == "STRING":
-                val = '"' + val + '"'
+               val = '"' + val + '"'
             elif not val:
                 val = '""'
             f.write(('BA_ "' + attrib + '" BO_ %d ' %
-                     bo.id + val + ';\n').encode(dbcExportEncoding))
+                     frame.id + val + ';\n').encode(dbcExportEncoding))
     f.write("\n".encode(dbcExportEncoding))
 
     # signal-attributes:
-    for bo in db.frames:
-        for signal in bo.signals:
+    for frame in db.frames:
+        for signal in frame.signals:
             for attrib, val in sorted(signal.attributes.items()):
-                name = normalizeName(signal.name, whitespaceReplacement)
+                name = output_names[frame][signal]
                 if db.signalDefines[attrib].type == "STRING":
                     val = '"' + val + '"'
                 elif not val:
                     val = '""'
+                elif isinstance(val, float):
+                    val = format_float(val)
                 f.write(
                     ('BA_ "' +
                      attrib +
                      '" SG_ %d ' %
-                     bo.id +
+                     frame.id +
                      name +
                      ' ' +
                      val +
                      ';\n').encode(dbcExportEncoding))
+            if signal.is_float:
+                if int(signal.signalsize) > 32:
+                    f.write(('SIG_VALTYPE_ %d %s : 2;\n' % (frame.id, output_names[bo][signal])).encode(dbcExportEncoding))
+                else:
+                    f.write(('SIG_VALTYPE_ %d %s : 1;\n' % (frame.id, output_names[bo][signal])).encode(dbcExportEncoding))
+ 
     f.write("\n".encode(dbcExportEncoding))
 
     # signal-values:
     for bo in db.frames:
+        multiplex_written = False
         for signal in bo.signals:
+            if signal.multiplex == 'Multiplexor' and multiplex_written:
+                continue
+
+            multiplex_written = True
+
             if signal.values:
                 f.write(
                     ('VAL_ %d ' %
                      bo.id +
-                     signal.name).encode(dbcExportEncoding))
+                     output_names[bo][signal]).encode(dbcExportEncoding))
                 for attrib, val in sorted(
                         signal.values.items(), key=lambda x: int(x[0])):
                     f.write(
@@ -333,7 +398,7 @@ def dump(db, f, **options):
             f.write(("SIG_GROUP_ " + str(bo.id) + " " + sigGroup.name +
                      " " + str(sigGroup.id) + " :").encode(dbcExportEncoding))
             for signal in sigGroup.signals:
-                f.write((" " + signal.name).encode(dbcExportEncoding))
+                f.write((" " + output_names[bo][signal]).encode(dbcExportEncoding))
             f.write(";\n".encode(dbcExportEncoding))
 
 
@@ -404,7 +469,7 @@ def load(f, **options):
             db._fl.addFrame(Frame(temp.group(2),
                                   Id=temp.group(1),
                                   dlc=temp.group(3),
-                                  transmitter=temp.group(4)))
+                                  transmitter=temp.group(4).split()))
         elif decoded.startswith("SG_ "):
             pattern = "^SG\_ (\w+) : (\d+)\|(\d+)@(\d+)([\+|\-]) \(([0-9.+\-eE]+),([0-9.+\-eE]+)\) \[([0-9.+\-eE]+)\|([0-9.+\-eE]+)\] \"(.*)\" (.*)"
             regexp = re.compile(pattern)
@@ -585,17 +650,18 @@ def load(f, **options):
                 botschaftId = temp.group(1)
                 signal = temp.group(2)
                 tempList = temp.group(3).split('"')
-                try:
-                    for i in range(math.floor(len(tempList) / 2)):
+                if botschaftId.isnumeric(): # value for Frame
+                    try:
                         bo = db.frameById(botschaftId)
                         sg = bo.signalByName(signal)
-                        val = tempList[i * 2 + 1]
-                        #[1:-1]
-
-                        if sg:
-                            sg.addValues(tempList[i * 2], val)
-                except:
-                    logger.error("Error with Line: " + str(tempList))
+                        for i in range(math.floor(len(tempList) / 2)):
+                            val = tempList[i * 2 + 1]
+                            if sg:
+                                sg.addValues(tempList[i * 2], val)
+                    except:
+                        logger.error("Error with Line: " + str(tempList))
+                else:
+                    logger.info("Warning: enviroment variables currently not supported")
 
         elif decoded.startswith("VAL_TABLE_ "):
             regexp = re.compile("^VAL\_TABLE\_ (\w+) (.*);")
@@ -652,7 +718,7 @@ def load(f, **options):
                                     temp_raw.group(2).decode(dbcImportEncoding))
 
         elif decoded.startswith("BA_ "):
-            regexp = re.compile("^BA\_ +\"[A-Za-z0-9\-_]+\" +(.+)")
+            regexp = re.compile("^BA\_ +\"[A-Za-z0-9[\-_ .]+\" +(.+)")
             tempba = regexp.match(decoded)
 
             if tempba.group(1).strip().startswith("BO_ "):
@@ -686,6 +752,16 @@ def load(f, **options):
             if frame is not None:
                 signalArray = temp.group(4).split(' ')
                 frame.addSignalGroup(temp.group(2), temp.group(3), signalArray)
+                
+        elif decoded.startswith("SIG_VALTYPE_ "):
+            regexp = re.compile("^SIG\_VALTYPE\_ +(\w+) +(\w+) +\:(.*);")
+            temp = regexp.match(decoded)
+            frame = db.frameById(temp.group(1))
+            if frame:
+                signal = frame.signalByName(temp.group(2))
+                signal.is_float = True
+#                SIG_VALTYPE_ 0 float : 1;
+                
         elif decoded.startswith("BA_DEF_DEF_ "):
             pattern = "^BA\_DEF\_DEF\_ +\"([A-Za-z0-9\-_]+)\" +(.+)\;"
             regexp = re.compile(pattern)
@@ -697,7 +773,7 @@ def load(f, **options):
                                     temp_raw.group(2).decode(dbcImportEncoding))
 #               else:
 #                       print "Unrecocniced line: " + l + " (%d) " % i
-
+# Backtracking
     for frame in db.frames:
         # receiver is only given in the signals, so do propagate the receiver
         # to the frame:
@@ -706,4 +782,24 @@ def load(f, **options):
         if frame.id > 0x80000000:
             frame.id -= 0x80000000
             frame.extended = 1
+    for define in db.globalDefines:
+        if db.globalDefines[define].type == "STRING":
+            if define in db.attributes:
+                db.attributes[define] = db.attributes[define][1:-1]
+    for define in db.buDefines:
+        if db.buDefines[define].type == "STRING":
+            for ecu in db.boardUnits:
+                if define in ecu.attributes:
+                    ecu.attributes[define] = ecu.attributes[define][1:-1]
+    for define in db.frameDefines:
+        if db.frameDefines[define].type == "STRING":
+            for frame in db.frames:
+                if define in frame.attributes:
+                    frame.attributes[define] = frame.attributes[define][1:-1]
+    for define in db.signalDefines:
+        if db.signalDefines[define].type == "STRING":
+            for frame in db.frames:
+                for signal in frame.signals:
+                    if define in signal.attributes:
+                        signal.attributes[define] = signal.attributes[define][1:-1]
     return db
